// Module Name: UART_RX_Example
// Target Device: EP4CE6E22C8N
// Tool versions: Quartus II 14.1
// Description: This example demonstrates a UART receiver on EP4CE6 Mini Board. The driver software for FT232R must be properly installed.
// You must have a serial port monitor software like Hyper Terminal to operate UART port.
// Port number could be found in Device Manager list after board is connected to PC via USB cable.
// UART parameters: baud rate: 19200, data width: 8 bits, parity check: none, stop bits: 1 bit
// After UART has been successfully configured and connected, you can send a 8-bit data in Hexadecimal format to UART. The data will be
// displayed on 8 on-board LEDs.

module UART_RX_Example( clk_50M, reset, rx, dataout);

input  wire clk_50M;
input  wire reset;
input  wire rx;

output wire[7:0] dataout;


wire clk16x;


// Baud Rate Generator
mod_m_counter #( .N(8), .M(161) ) baud_rate_gen	(	.clk(clk_50M),
																	.reset(reset),
																	.max_tick(clk16x),
																	.q()
																 );
// UART Receiver
uart_rx rxd( .reset(reset), 
				 .clk16x(clk16x), 
				 .rbr(dataout), 
				 .rbr_rd_strobe( ),
				 .lse_rd_strobe(),				 
				 .databits(2'b11), // 8 data bits
				 .parity_en(1'b0), // disable parity check
				 .parity_even(1'b0), // ignored when parity check disabled
				 .parity_stick(1'b0), //ignore when parity check disabled
				 .sin(rx), 
				 .rx_rdy(), 
				 .err_overrun(), 
				 .err_parity(),
				 .err_frame(), 
				 .err_break_int()
			  );
																	  
																	  
endmodule
