`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    16:07:16 12/30/2014 
// Design Name: 
// Module Name:    uart_rx 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////

module uart_rx( reset, clk16x, rbr, rbr_rd_strobe,lse_rd_strobe, databits, parity_en, parity_even, parity_stick, sin, rx_rdy, err_overrun, err_parity, err_frame, err_break_int );

input  wire reset;			// global reset
input  wire clk16x;			// receiver module clock,16 times buadrate
input  wire rbr_rd_strobe;	// rbr read strobe signal
input  wire lse_rd_strobe;
input  wire parity_en;		// parity enable 
input  wire parity_even;	// 0--odd parity check,1--even parity check
input  wire parity_stick;	// stick parity
input  wire sin;				// serial data input
input  wire [1:0] databits;// set databits--5,6,7,8

output wire [7:0] rbr;		// receiver buffer register
output wire rx_rdy;			// data ready to read
output wire err_overrun;	// overrun error
output wire err_parity;		// parity error
output wire err_frame;		// frame error
output wire err_break_int;


reg [3:0] rx_bit_cnt;		// number of bits received
reg [3:0] count;				// 
reg [7:0] rsr;					// receiver shift register
reg [7:0] rbr_reg;
reg rx_err_parity;			// receiver parity error
reg rx_err_frame;				// receiver frame error
reg rx_idle;					// 
reg rbr_data_ready;			//
reg err_overrun_reg;
reg err_parity_reg;
reg err_frame_reg;
reg err_break_int_reg;

reg hunt;
reg huntone;

reg sampledonce;
reg rx_clken;	// receiver clock enable

reg sin1;
reg rx_idle1;
reg rx_err_frame1;

//state definition
parameter	idle		=  2'b00;
parameter	shift		=  2'b01;
parameter	parity	=  2'b10;
parameter	stop		=  2'b11;

reg [1:0] rx_state;
 
//generate rx_clken signal
//rx_clken:serial data receiving clock enable
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		rx_clken <= 1'b0;
	else
		begin
			if( count == 4'b0110 )
				rx_clken <= 1'b1;
			else
				rx_clken <= 1'b0;
		end
end

//count:4-bit counter for rx_clken waveform generation
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		count <= 4'b0000;
	else
		begin
			if( ( rx_state != idle )||( hunt == 1'b1 ) )
				count <= count + 1'b1;
			else if( sampledonce == 1'b1 )
				count <= 4'b0010;
			else
				count <= 4'b0000;
		end
end

//generate hunt
//hunt will be true when start bit is found
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		hunt <= 1'b0;
	else 
		begin
			if( ( rx_state == idle )&&( sin == 1'b0 )&&( sin1 == 1'b1 ) )
				hunt <= 1'b1; // Set Hunt_r when SIN falling edge is found at the idle state
			else if( ( sampledonce == 1'b1 )&&( sin == 1'b0 ) ) // Start bit is successfully sampled twice after framing error
				hunt <= 1'b1; // set hunt "true" for resynchronizing of next frame
			else if( ( rx_idle == 1'b0 )||( sin == 1'b1 ) )
				hunt <= 1'b0; // Clear hunt when data shifting starts or when SIN returns to '1'
		end
end


//huntone
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		huntone <= 1'b0;
	else 
		begin
			if( hunt == 1'b1 )
				huntone <= 1'b0;
			else if( ( rx_idle == 1'b0 )&&( count[3] == 1'b1 )&&( sin == 1'b1 ) )
				huntone <= 1'b1;
		end
end

//rbr data ready
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		rbr_data_ready <= 1'b0;
	else
		begin
			if( ( rx_idle == 1'b1 )&&( rx_idle1 == 1'b0 ) )
				rbr_data_ready <= 1'b1; // set RbrDataRDY at rx_idle rising edge
			else if( rbr_rd_strobe == 1'b1 )
				rbr_data_ready <= 1'b0; // clear rbr_data_ready when RBR is read by CPU
		end
end

//sampledonce
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		sampledonce <= 1'b0;
	else
		begin
			if( ( rx_err_frame == 1'b1 )&&( rx_err_frame1 == 1'b0 )&&( sin == 1'b0 )&&( huntone == 1'b1 ) )
				sampledonce <= 1'b1;// Start bit got sampled once
			else
				sampledonce <= 1'b0;
		end
end

//rx_idle flag
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		rx_idle <= 1'b1;
	else
		begin
			if( rx_state == idle )
				rx_idle <= 1'b1;
			else if( count[3] == 1'b1 )
				rx_idle <= 1'b0;
		end
end


//receiver finite machine
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		begin
			rsr <= 8'b0000_0000;
			rx_bit_cnt <= 4'b0000;
			rx_err_parity <= 1'b1;
			rx_err_frame  <= 1'b0;
			rx_state <= idle;
		end
	else
		begin
			case( rx_state )
				idle:
					if( ( rx_idle == 1'b1 )&&( sin == 1'b0 )&&( rx_clken == 1'b1 ) )
						begin
							rsr <= 8'b0000_0000;
							rx_bit_cnt <= 4'b0000;
							rx_err_parity <= ~parity_even;
							rx_err_frame <= 1'b0;
							rx_state <= shift;
						end
					
				shift:
					if( rx_clken == 1'b1 )
						begin
							rsr <= { sin,rsr[7:1] };
							rx_err_parity <= rx_err_parity ^ sin;
							rx_bit_cnt <= rx_bit_cnt + 1'b1;
							if( ( databits == 2'b00 && rx_bit_cnt == 4 ) ||
								 ( databits == 2'b01 && rx_bit_cnt == 5 ) ||
								 ( databits == 2'b10 && rx_bit_cnt == 6 ) ||
								 ( databits == 2'b11 && rx_bit_cnt == 7 ) 
								)
								begin
									if( parity_en == 1'b0 )
										rx_state <= stop;
									else
										rx_state <= parity;
								end
						end
							 
				parity:
					if( rx_clken == 1'b1 )
						begin
							if( parity_stick == 1'b0 )
								rx_err_parity <= rx_err_parity ^ sin;
							else
								begin
									if( parity_even == 1'b0 )
										rx_err_parity <= !sin;
									else
										rx_err_parity <= sin;
								end
							rx_state <= stop;
						end
						
				stop:
					if( rx_clken == 1'b1 )
						begin
							rx_err_frame <= !sin;
							rx_state <= idle;
						end
				default:
					if( rx_clken == 1'b1 )
						rx_state <= idle;
						
			endcase
		end
end
		
		
//receiver buffer register
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		rbr_reg <= 8'b0000_0000;
	else
		begin
			if( ( rx_idle == 1'b1 )&&( rx_idle1 == 1'b0 ) )
				case( databits )
					2'b00: rbr_reg <= { 3'b000,rsr[7:3] };
					2'b01: rbr_reg <= { 2'b00,rsr[7:2] };
					2'b10: rbr_reg <= { 1'b0,rsr[7:1] };
					default:rbr_reg <= rsr;
				endcase
		end
end

assign rbr = rbr_reg;

//delayed signals for edge detection
always@( posedge clk16x, posedge reset )
begin
	if( reset == 1'b1 )
		begin
			sin1 <= 1'b0;
			rx_err_frame1 <= 1'b1;
			rx_idle1 <= 1'b1;
		end
	else
		begin
			sin1 <= sin;
			rx_err_frame1 <= rx_err_frame;
			rx_idle1 <= rx_idle;
		end
end


//generate erroe flags
always@( posedge clk16x,  posedge reset )
begin
	if( reset == 1'b1 )
		begin
			err_overrun_reg <= 1'b0;
			err_parity_reg <= 1'b0;
			err_frame_reg <= 1'b0;
			err_break_int_reg <= 1'b0;
		end
	else
		begin
			if( ( rx_idle == 1'b1 )&&( rx_idle1 == 1'b0 ) )
				begin
					err_overrun_reg <= rbr_data_ready;
					err_parity_reg <= ( err_parity_reg | rx_err_parity ) && parity_en;
					err_frame_reg <= err_frame_reg | rx_err_frame;
					err_break_int_reg <= err_break_int_reg | ( ~huntone );
				end				
			else if( lse_rd_strobe == 1'b1 )
				begin
					err_overrun_reg <= 1'b0;
					err_parity_reg <= 1'b0;
					err_frame_reg <= 1'b0;
					err_break_int_reg <= 1'b0;
				end
		end

end

//receiver ready for read when data is available in rbr
assign rx_rdy = rbr_data_ready;
assign err_overrun = err_overrun_reg;
assign err_parity = err_parity_reg;
assign err_frame = err_frame_reg;
assign err_break_int = err_break_int_reg;



endmodule
